/**
 * Copyright (C) 2001-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This  demo  illustrates how  to  use  Foxit PDF SDK  to mark  PDF content.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfcontentmark
#endif

#define  PageWidth             612                      //The new page width
#define  PageHeight            792                      //The new page height


void InsertGenerateContents(FSCRT_PAGE page, FSPDF_PAGEOBJECTS pageObjs, FS_INT32 typeFilter, FS_INT32 index, FSPDF_PAGEOBJECT pageObj)
{
	//Insert a page object
	FS_RESULT ret = FSPDF_PageObjects_InsertObject(page, pageObjs, typeFilter, index, pageObj);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_PageObjects_InsertObject with error code %d.\r\n", ret);
		return;
	}

	ret = FSPDF_PageObjects_GenerateContents(page, pageObjs);	
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_PageObjects_GenerateContents with error code %d.\r\n", ret);
		return;
	}
	FSDK_OutputLog("Success: Generate PDF page contents.\r\n");
}
FS_RESULT GetMarkedContenObject(string mcFile)
{
	CAutoDocument autoDoc(mcFile.c_str(), "rb");
	if (!autoDoc.GetFileHander()) return FSCRT_ERRCODE_ERROR;
	if (FSCRT_ERRCODE_SUCCESS != autoDoc.LoadPDFDocument(NULL))
		return FSCRT_ERRCODE_ERROR;

	CAutoPDFPage autopage;
	FS_RESULT ret = autopage.LoadPDFPage(autoDoc.GetDocument(), 0);
	if (FSCRT_ERRCODE_SUCCESS != ret) return ret;
	autopage.ParsePage();

	FSPDF_PAGEOBJECTS pageObjs = NULL;
	ret = FSPDF_Page_GetPageObjects(autopage.GetPDFPage(), &pageObjs);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Page_GetPageObjects with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
		return ret;
	}

	FSPDF_PAGEOBJECT pageObj = NULL;
	ret = FSPDF_PageObjects_GetObject(autopage.GetPDFPage(), pageObjs, FSPDF_PAGEOBJECT_TEXT, 0,&pageObj);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_PageObjects_GetObject with error code %d.\r\n", ret);
		return ret;
	}

	FSPDF_MARKEDCONTENT mc = NULL;
	ret = FSPDF_PageObject_GetMarkedContent(autopage.GetPDFPage(), pageObj, &mc);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_PageObject_GetMarkedContent with error code %d.\r\n", ret);		
		return ret;
	}
	FSDK_OutputLog("Success: Get a marked content.\r\n");

	FS_INT32 count;
	ret = FSPDF_MarkedContent_CountItems(autopage.GetPDFPage(), mc, &count);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_MarkedContent_CountItems with error code %d.\r\n", ret);		
		return ret;
	}
	FSCRT_BSTR tagName;
	FSCRT_BStr_Init(&tagName);
	ret = FSPDF_MarkedContent_GetTagName(autopage.GetPDFPage(), mc, 0, &tagName);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_MarkedContent_GetTagName with error code %d.\r\n", ret);		
		return ret;
	}
	FS_INT32 paramType;
	FSPDF_OBJECT obj;
	ret = FSPDF_MarkedContent_GetItemParam(autopage.GetPDFPage(), mc, &tagName, &paramType, (FS_LPVOID*)&obj);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_BStr_Clear(&tagName);
		FSDK_OutputLog("Failure: In function FSPDF_MarkedContent_CountItems with error code %d.\r\n", ret);		
		return ret;
	}

	string strType = "";
	switch (paramType)
	{
	case FSPDF_MARKEDCONTENT_PARAMTYPE_NONE:
		{
			strType = "NONE";
		}
		break;
	case FSPDF_MARKEDCONTENT_PARAMTYPE_PROPERTIESDICT:
		{
			strType = "PROPERTIESDICT";
		}
		break;
	case FSPDF_MARKEDCONTENT_PARAMTYPE_DIRECTDICT:
		{
			strType = "DIRECTDICT";
		}
		break;
	case FSPDF_MARKEDCONTENT_PARAMTYPE_MCID:
		{
			strType = "MCID";
		}
		break;
	}
	
	FSDK_OutputLog("Success: The MarkedContent item tag name is %s and the MarkedContent type is %s.\r\n", tagName.str, strType.c_str());
	FSCRT_BStr_Clear(&tagName);
	FSDK_OutputLog("Success: Get Marked Content Object.\r\n");
	return FSCRT_ERRCODE_SUCCESS;
}

FSPDF_PAGEOBJECT AddTextObject(FSCRT_PAGE page, FSPDF_PAGEOBJECTS pageObjs)
{
	FSPDF_PAGEOBJECT textObj = NULL;
	FS_RESULT ret = FSPDF_TextObject_Create(page, &textObj);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_TextObject_Create with error code %d.\r\n", ret);
		return NULL;
	}
	FSCRT_BSTR fontName;
	FSCRT_BStr_Init(&fontName);
	fontName.str = (FS_LPSTR)"Helvetica";
	fontName.len = strlen(fontName.str);
	FSCRT_FONT creatFont = NULL;
	ret = FSCRT_Font_Create(&fontName, FSCRT_FONTSTYLE_SERIF,100, 0, &creatFont);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSPDF_PageObject_Release(page, textObj);
		FSDK_OutputLog("Failure: In function FSCRT_Font_Create with error code %d.\r\n", ret);
		return NULL;
	}
	FSPDF_TEXTSTATE textState;
	textState.charSpace = 0;
	textState.font = creatFont;
	textState.fontSize = 50;
	textState.textMatrix[0] = 4;
	textState.textMatrix[1] = 10;
	textState.textMatrix[2] = 1;
	textState.textMatrix[3] = 10;
	textState.textMode = 0;
	textState.wordSpace = 0;
	textState.x = 0;
	textState.y = 0;
	ret = FSPDF_TextObject_SetTextState(page, textObj, &textState, TRUE, 400);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_Font_Release(creatFont);
		FSPDF_PageObject_Release(page, textObj);
		FSDK_OutputLog("Failure: In function FSPDF_TextObject_SetTextState with error code %d.\r\n", ret);
		return NULL;
	}
	char*  TextObjUnicodeStr = (char*)"Insert a Text Object";
	FSCRT_BSTR unicodeStr = {TextObjUnicodeStr, (FS_DWORD)strlen(TextObjUnicodeStr)};
	ret = FSPDF_TextObject_SetUnicodeString(page,textObj,&unicodeStr);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_Font_Release(creatFont);
		FSPDF_PageObject_Release(page, textObj);
		FSDK_OutputLog("Failure: In function FSPDF_TextObject_SetUnicodeString with error code %d.\r\n", ret);
		return NULL;
	}
	FSCRT_MATRIX textmatrix = {1, 0, 0, 1, 100, 300};
	ret = FSPDF_PageObject_SetMatrix(page, textObj, &textmatrix);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_Font_Release(creatFont);
		FSPDF_PageObject_Release(page, textObj);
		FSDK_OutputLog("Failure: In function FSPDF_PageObject_SetMatrix with error code %d.\r\n", ret);
		return NULL;
	}
	FSCRT_Font_Release(creatFont);
	//Insert the text object and generate PDF page contents.
	ret = FSPDF_PageObjects_InsertObject(page, pageObjs, FSPDF_PAGEOBJECT_TEXT, 0, textObj);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSPDF_PageObject_Release(page, textObj);
		FSDK_OutputLog("Failure: In function FSPDF_PageObjects_InsertObject with error code %d.\r\n", ret);
		return NULL;
	}

	ret = FSPDF_PageObjects_GenerateContents(page, pageObjs);	
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_PageObjects_GenerateContents with error code %d.\r\n", ret);
		return NULL;
	}

	FSDK_OutputLog("Success: Add text object.\r\n");
	return textObj;
}
FS_RESULT CreateMarkedContent(string outputFilesPath)
{
	CAutoDocument autoDoc;
	FS_RESULT ret = autoDoc.CreatePDFDocument();
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	CAutoPDFPage autoPage;
	ret = autoPage.CreatePDFPage(autoDoc.GetDocument(), 0);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	ret = FSPDF_Page_SetSize(autoPage.GetPDFPage(), PageWidth, PageWidth);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Page_SetSize with error code %d.\r\n", ret);
		return ret;
	}

	FSPDF_PAGEOBJECTS pageObjs = NULL;
	ret = FSPDF_Page_GetPageObjects(autoPage.GetPDFPage(), &pageObjs);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Page_GetPageObjects with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
		return ret;
	}
	FSDK_OutputLog("Success: Get page objects in PDF page.\r\n");

	FSPDF_PAGEOBJECT pageObj = AddTextObject(autoPage.GetPDFPage(), pageObjs);
	if (!pageObj) return FSCRT_ERRCODE_ERROR;
	
	//set marked content
	FSPDF_MARKEDCONTENT mc;
	ret = FSPDF_PageObject_GetMarkedContent(autoPage.GetPDFPage(), pageObj, &mc);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_PageObject_GetMarkedContent with error code %d.\r\n", ret);
		return ret;
	}
	FS_INT32 count;
	ret = FSPDF_MarkedContent_CountItems(autoPage.GetPDFPage(), mc, &count);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_MarkedContent_CountItems with error code %d.\r\n", ret);
		return ret;
	}
	FSDK_OutputLog("Success: Get marked content object from page object.\r\n");
	FSCRT_BSTR tagName;
	FSCRT_BStr_Init(&tagName);
	tagName.str = (FS_LPSTR)"Span";
	tagName.len = strlen("Span");
	FSPDF_OBJECT object = NULL;
	ret = FSPDF_Object_CreateDict(autoDoc.GetDocument(), &object);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Object_CreateDict with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
		return ret;
	}
	FSCRT_BSTR strKye;
	FSCRT_BStr_Init(&strKye);
	FSCRT_BStr_Set(&strKye, "Direct",strlen("Direct"));
	ret = FSPDF_Dictionary_SetAtInteger(autoDoc.GetDocument(), object, &strKye, 55);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_BStr_Clear(&strKye);
		FSPDF_Object_Release(autoDoc.GetDocument(), object);
		FSDK_OutputLog("Failure: In function FSPDF_MarkedContent_CountItems with error code %d.\r\n", ret);
		return ret;
	}
	FSDK_OutputLog("Success: Create a dictionary PDF object.\r\n");
	ret = FSPDF_MarkedContent_AddItem(autoPage.GetPDFPage(), mc, &tagName, FSPDF_MARKEDCONTENT_PARAMTYPE_DIRECTDICT, (FS_LPVOID*)object);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_BStr_Clear(&strKye);
		FSPDF_Object_Release(autoDoc.GetDocument(), object);
		FSDK_OutputLog("Failure: In function FSPDF_MarkedContent_AddItem with error code %d.\r\n", ret);
		return ret;
	}
	FSDK_OutputLog("Success: Add object of marked content item to the current marked content.\r\n");
	ret = FSPDF_PageObjects_GenerateContents(autoPage.GetPDFPage(), pageObjs);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_BStr_Clear(&strKye);
		FSPDF_Object_Release(autoDoc.GetDocument(), object);
		FSDK_OutputLog("Failure: In function FSPDF_PageObjects_GenerateContents with error code %d.\r\n", ret);
		return ret;
	}
	FSDK_OutputLog("Success: Generate PDF Page content.\r\n");
	string savePath = outputFilesPath + "/" + "MarkedContent.pdf";
	ret = FSDK_SavePDFFile(autoDoc.GetDocument(), savePath.c_str());
	if (ret == false) return FSCRT_ERRCODE_ERROR;

	FSPDF_Object_Release(autoDoc.GetDocument(), object);
	FSCRT_BStr_Clear(&strKye);
	return FSCRT_ERRCODE_SUCCESS;
}

void AddMarkedContentFolder(void* folder, string inputFilesPath, string outputFilesPath)
{
	do
	{//Enumerate files in input file folder and choose PDF file to load
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if (strFile.size() > 0 && !isDir)
		{
			if (FSDK_CompareExtName(strFile.c_str(), ".pdf"))
			{
				CreateMarkedContent(outputFilesPath);
				string mcDocString = outputFilesPath + "/" + "MarkedContent.pdf";
				GetMarkedContenObject(mcDocString);
			}
		}
		if (strFile.size() < 1) break;
	} while (1);
}

int main(int argc,char* argv[])
{
	FSDK_SetExecuteFilePath(argc, argv);
	string logPath = FSDK_GetLogFile("pdfcontentmark");
	//Open and write log
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfcontentmark\r\n\r\n");
	FSDK_OutputLog("\tInput from: input_files folder\r\n");
	FSDK_OutputLog("\tOutput to: output_files/pdfcontentmark\r\n\r\n");

	//Initialize SDK library, or no method of SDK can be used
	bool bRet = FSDK_InitializeLibray(false);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module, or no method of PDF module can be used
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}
	
	string outputFilesPath = FSDK_GetOutputFilesFolder("pdfcontentmark");

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFilesPath + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	CreateMarkedContent(outputFilesPath);
	string mcDocString = outputFilesPath + "/" + "MarkedContent.pdf";
	GetMarkedContenObject(mcDocString);


	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;

}


