#include "signature.h"

#define CLEAR_FSBSTR(bstr) if (bstr.len && bstr.str)FSCRT_BStr_Clear(&bstr)

#define VALIDATE_PARAM(index, argv, arraySize)	if (index >= arraySize-1)return 1; \
												if(argv[index + 1][0] == '/'){i+=1; continue;}

FS_BOOL parseParam(int argc, char *argv[], TIMESTAMP_PARAM *out)
{
	FS_RESULT ret = 0;
	int i = 1;
	while(i < argc)
	{
		if (0 == strcmp(argv[i],"/i"))
		{
			VALIDATE_PARAM(i, argv, argc);
			FSCRT_BStr_Set(&out->inputFile,argv[i+1], strlen(argv[i+1]));
			i+=2;
		} 
		else  if(0 == strcmp(argv[i],"/o"))
		{
			VALIDATE_PARAM(i, argv, argc);
			string outputPath = argv[i + 1];
			outputPath += "/pdfdigitalsignature";
			FSCRT_BStr_Set(&out->outputFile,argv[i+1], strlen(argv[i+1]));
			i+=2;
		} 
		else  if(0 == strcmp(argv[i],"/sf"))
		{
			VALIDATE_PARAM(i, argv, argc);
			if (strcmp(argv[i + 1], "adbe.pkcs7.sha1") != 0 && strcmp(argv[i + 1], "adbe.pkcs7.detached") != 0 && strcmp(argv[i + 1], "ETSI.RFC3161") != 0)
			{
				printf("unspported subfilter: %s\r\n", argv[i+1]);
				return 0;
			}
			FSCRT_BStr_Set(&out->subfilter,argv[i+1], strlen(argv[i+1]));
			i+=2;
		} 
		else if(0 == strcmp(argv[i],"/t"))
		{
			VALIDATE_PARAM(i, argv, argc);
			if (0 == strcmp("doc", argv[i + 1]))
			{
				out->TSType = TIMESTAMP_DOC;
			}
			else if (0 == strcmp("sig", argv[i + 1]))
			{
				out->TSType = TIMESTAMP_SIG;
			}
			else {
				printf("not supported option: %s %s",argv[i], argv[i + 1]);
				return 0;
			}
			i+=2;
		} 
		else if(0 == strcmp(argv[i],"/u"))
		{
			VALIDATE_PARAM(i, argv, argc);
			string url = argv[i + 1];
			if (url.find("http")!= -1)
				url = url.substr(url.find("//") + 2);

			if (url.find("/") ==-1)
				return 0;
			string host = url.substr(0,url.find("/"));
			ret = FSCRT_BStr_Set(&out->tsaHost, host.c_str(), host.length());
			string tsaReq = url.substr(url.find("/"));
			ret = FSCRT_BStr_Set(&out->tsaReqUrl, tsaReq.c_str(), tsaReq.length());
			i+=2;
		} 
		else if(0 == strcmp(argv[i],"/up"))
		{
			VALIDATE_PARAM(i, argv, argc);
			FSCRT_Base64_EncodeFromBuffer(&out->tsaUserNPswd, argv[i + 1], strlen(argv[i + 1]));
			i+=2;
		} 
		else if(0 == strcmp(argv[i],"/p"))
		{
			VALIDATE_PARAM(i, argv, argc);
			out->port = atoi(argv[i + 1]);
			i+=2;
		} 
		else
			return 0;
	}
	return 1;
}

//we use adbe.pkcs7.sha1 to sign a document, and if need a time stamp, sha1 will be used.
FS_BOOL validateParam(TIMESTAMP_PARAM *param)
{
	FS_RESULT nRet = 0;

	string pdffile;
	if (param->inputFile.len == 0)
	{
		pdffile = "AboutFoxit.pdf";
		string inputFilesPath = FSDK_GetFixFolder();
		string file = inputFilesPath +"/"+ pdffile;
		nRet = FSCRT_BStr_Set(&param->inputFile, file.c_str(), file.length());
	}
	else
	{
		FILE *input = fopen(param->inputFile.str,"rb");
		if (!input)
		{
			printf("can't open input file: %s", param->inputFile.str);
			return FALSE;
		}
		fclose(input);
	}

	if (param->outputFile.len == 0)
	{
		pdffile = FSDK_GetFileName(param->inputFile.str);
		//Get output files folder path.
		string outputFilesPath = FSDK_GetOutputFilesFolder("pdfdigitalsignature");
		string outputFile = outputFilesPath+"/" + pdffile;
		nRet = FSCRT_BStr_Set(&param->outputFile, outputFile.c_str(), outputFile.length());
	}
	else
	{
		pdffile = FSDK_GetFileName(param->inputFile.str);
		string outputFile = param->outputFile.str;
		outputFile += "/pdfdigitalsignature/";
		FSDK_CreatFolder(outputFile.c_str());
		outputFile += pdffile;
		nRet = FSCRT_BStr_Set(&param->outputFile, outputFile.c_str(), outputFile.length());
	}
	if (param->certFile.len == 0)
	{
		string inputFolder = FSDK_GetFixFolder();
		inputFolder += "/foxit_all.pfx";
		nRet = FSCRT_BStr_Set(&param->certFile, inputFolder.c_str(), inputFolder.length());
		if (nRet == FSCRT_ERRCODE_SUCCESS)
		{
			const char* password = "123456";
			nRet = FSCRT_BStr_Set(&param->certPwd, password, strlen(password));
		}
	}
	else
	{
		FILE *input = fopen(param->certFile.str,"rb");
		if (!input)
		{
			printf("can't open input file: %s", param->certFile.str);
			return FALSE;
		}
		fclose(input);
	}

	int tsTypeAvail = 1;
	if (param->subfilter.len == 0)//user did not specify the subfilter
	{
		nRet = FSCRT_BStr_Set(&param->subfilter, "adbe.pkcs7.sha1", strlen("adbe.pkcs7.sha1"));
	}
	else if (param->subfilter.len != 0)
	{
		if(strcmp(param->subfilter.str, "ETSI.RFC3161") == 0)
		{
			if (param->TSType != TIMESTAMP_DOC)
			tsTypeAvail=0;
		}
		else if (param->TSType == TIMESTAMP_DOC)
				tsTypeAvail=0;
	}

	if(!tsTypeAvail)
	{
		if (param->TSType == TIMESTAMP_SIG)
			printf("the subfilter %s can't be used to time stamp a signature (specified by \"/t sig\").\r\n", param->subfilter.str);
		else 
			printf("the subfilter %s can't be used to time stamp a document (specified by \"/t doc\").\r\n", param->subfilter.str);
		return FALSE;
	}

	if (param->TSType != 0)
	{
		if (param->tsaHost.len == 0 || param->tsaReqUrl.len == 0){
			printf("TSA url should be specified");
			return FALSE;
		}
	}

	if (param->port == 0)param->port = 80;
	return TRUE;
}

void switchSubfilter(TIMESTAMP_PARAM*param)
{
	FS_RESULT nRet;
	if (param->TSType == TIMESTAMP_DOC)
	{
		param->TSType = TIMESTAMP_SIG;
		nRet = FSCRT_BStr_Set(&param->subfilter, "adbe.pkcs7.detached", strlen("adbe.pkcs7.detached"));
	}
	else if (param->TSType == TIMESTAMP_SIG || param->TSType == 0)
	{
		if (strcmp(param->subfilter.str, "adbe.pkcs7.sha1") == 0)
			nRet = FSCRT_BStr_Set(&param->subfilter, "adbe.pkcs7.detached", strlen("adbe.pkcs7.detached"));
		else
			nRet = FSCRT_BStr_Set(&param->subfilter, "adbe.pkcs7.sha1", strlen("adbe.pkcs7.sha1"));
	}

	nRet = FSCRT_BStr_Set(&param->inputFile, param->outputFile.str, param->outputFile.len);

	string outputFile = param->outputFile.str;
	string folder = outputFile.substr(0,outputFile.rfind("/")+1);
	string newOutpuFile = folder + "newSubfilter_" + FSDK_GetFileName(param->outputFile.str);
	nRet = FSCRT_BStr_Set(&param->outputFile, newOutpuFile.c_str(), newOutpuFile.length());
}

void clearParam(TIMESTAMP_PARAM *TSParam)
{
	CLEAR_FSBSTR(TSParam->inputFile);
	CLEAR_FSBSTR(TSParam->outputFile);
	CLEAR_FSBSTR(TSParam->certFile);
	CLEAR_FSBSTR(TSParam->certPwd);
	CLEAR_FSBSTR(TSParam->tsaHost);
	CLEAR_FSBSTR(TSParam->tsaReqUrl);
	CLEAR_FSBSTR(TSParam->tsaUserNPswd);
	CLEAR_FSBSTR(TSParam->subfilter);
	delete TSParam->pageRange;
}
