/**
 * Copyright (C) 2001-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is pdf page organization source to implement page organization examples.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfpage_organization
#endif

//Set page index
FS_RESULT SetIndex_Page(FSCRT_DOCUMENT srcDoc)
{
	FS_RESULT ret;
	FSCRT_PAGE page = NULL;
	ret = FSPDF_Doc_GetPage(srcDoc,0,&page);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: Fail to get page with index 0, with error code %d.\r\n", ret);
		return ret;
	}
	FSDK_OutputLog("Success: Get page with index 0.\r\n");
	ret = FSPDF_Page_SetIndex(page, 2);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Page_SetIndex with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
	}
	else
		FSDK_OutputLog("Success: Set page index from 0 to 2 successfully.\r\n");
	ret = FSPDF_Page_Clear(page);
	return ret;
}

//Delete src page
FS_RESULT Delete_Page(FSCRT_DOCUMENT srcDoc)
{
	FS_RESULT ret;
	FSCRT_PAGE page = NULL;
	ret = FSPDF_Doc_GetPage(srcDoc,1,&page);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: Fail to get page with index 1, with error code %d.\r\n", ret);
		return ret;
	}
	FSDK_OutputLog("Success: Get page with index 1.\r\n");
	ret = FSPDF_Page_Delete(page);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: Fail to delete page with index 1, with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
	}
	FSDK_OutputLog("Success: Delete page with index 1.\r\n");
	return ret;
}

//Insert pages in srcDoc to insertDoc.
FS_RESULT InsertPagesToDoc(FSCRT_DOCUMENT insertDoc, FSCRT_DOCUMENT srcDoc)
{
	FSDK_OutputLog("Insert pages to doc begin.\r\n");
	FS_RESULT ret;
	FSCRT_PROGRESS importProgress = NULL;
	FS_INT32* pageRanges = NULL;
	FS_INT32 page_count = 0;
	//Init pageRanges.
	FSPDF_Doc_CountPages(srcDoc, &page_count);
	pageRanges = (FS_INT32*)malloc(sizeof(FS_INT32) * 2);
	pageRanges[0] = 0;
	pageRanges[1] = page_count;

	//Insert pages to doc.
	ret = FSPDF_Doc_StartImportPages(insertDoc, 1, srcDoc, pageRanges, 2 , &importProgress);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: Fail to insert pages to document with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
	}
	ret = FSCRT_ERRCODE_TOBECONTINUED;
	while (FSCRT_ERRCODE_TOBECONTINUED == ret)
		ret = FSCRT_Progress_Continue(importProgress, NULL);
	if (FSCRT_ERRCODE_FINISHED != ret)
		FSDK_OutputLog("Failure: Fail to insert pages to document with error code %d.\r\n", ret);
	else
		FSDK_OutputLog("Success: Insert pages to document successfully.\r\n");
	//Release resources.
	FSCRT_Progress_Release(importProgress);
	importProgress = NULL;
	free(pageRanges);
	pageRanges = NULL;
	return FSCRT_ERRCODE_SUCCESS;
}

//Split srcDoc.
void Split_Doc(FSCRT_DOCUMENT srcDoc, string outputFile, string strFileName)
{
	FSDK_OutputLog("Split doc begin.\r\n");
	FS_RESULT ret;
	FSCRT_PROGRESS importProgress ;
	FS_INT32 pageRanges[2];
	//FSCRT_DOCUMENT newDoc;
	FS_INT32 page_count = 0;

	FSPDF_Doc_CountPages(srcDoc, &page_count);
	for (int i = 0; i < page_count; i++)
	{
		pageRanges[0] = i;
		pageRanges[1] = 1;
		//Create new doc.
		CAutoDocument autoNewDoc;
		ret =	autoNewDoc.CreatePDFDocument();
		if (ret != FSCRT_ERRCODE_SUCCESS)
			return;

		//Split each page to a new doc.
		ret = FSPDF_Doc_StartImportPages(autoNewDoc.GetDocument(), 0, srcDoc, pageRanges, 2, &importProgress);
		ret = FSCRT_ERRCODE_TOBECONTINUED;
		while (FSCRT_ERRCODE_TOBECONTINUED == ret)
			ret = FSCRT_Progress_Continue(importProgress, NULL);

		//Construct save file name and save.
		char* newFileName = (char*)malloc(256);

		memset(newFileName, 0, sizeof(char)*256);
		sprintf(newFileName, "_split_page[%d]", i);

		string savePath = outputFile + strFileName + newFileName + ".pdf";
		free(newFileName);
		newFileName = NULL;

		if (FSCRT_ERRCODE_FINISHED == ret)
		{
			FSDK_SavePDFFile(autoNewDoc.GetDocument(), savePath.c_str());
		}
		else
			FSDK_OutputLog("Failure: In function FSPDF_Doc_StartImportPages(Continue) with error code %d\r\n",ret);

		FSCRT_Progress_Release(importProgress);
		importProgress = NULL;
	}
	FSDK_OutputLog("Success: Split doc Finished.\r\n");
}

//Merge srcDoc to insertDoc.
void Merge_Doc(FSCRT_DOCUMENT srcDoc, FSCRT_DOCUMENT insertDoc)
{
	FSDK_OutputLog("Merge two doc begin.\r\n");
	FS_RESULT ret;
	FSCRT_PROGRESS importProgress = NULL;
	FS_INT32* pageRanges = NULL;
	FS_INT32 page_count = 0;

	//Init pageRanges.
	FSPDF_Doc_CountPages(srcDoc, &page_count);
	pageRanges = (FS_INT32*)malloc(page_count * sizeof(FS_INT32) * 2);
	for (int i = 0; i < page_count * 2; i+=2)
	{
		pageRanges[i] = i / 2;
		pageRanges[i+1] = 1;
	}

	//Merge two doc.
	//ret = FSCRT_ERRCODE_TOBECONTINUED;
	ret = FSPDF_Doc_StartImportPages(insertDoc, 0, srcDoc, pageRanges, page_count * 2, &importProgress);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Doc_StartImportPages with error code %d.\r\n", ret);
		return;
	}
	ret = FSCRT_ERRCODE_TOBECONTINUED;
	while (FSCRT_ERRCODE_TOBECONTINUED == ret)
		ret = FSCRT_Progress_Continue(importProgress, NULL);
	if (FSCRT_ERRCODE_FINISHED == ret ||  FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("Success: Merge two document successfully.\r\n");
	else
		FSDK_OutputLog("Failure: Merge two document failed, and error code %d.\r\n", ret);

	//Release resources.
	FSCRT_Progress_Release(importProgress);
	importProgress = NULL;
	free(pageRanges);
	pageRanges = NULL;
}

void InsertFilePage(string strFile, string inputFilesPath, string outputFile)
{
	int pos = (int)strFile.find_first_of('.');
	string strFileName = strFile;
	strFileName = strFileName.erase(pos, 4);

	for (int icount = 0; icount < 5; icount++)
	{
		FS_RESULT ret = FSCRT_ERRCODE_ERROR ;
		//Open source doc.
		FSDK_OutputLog("\r\n");
		CAutoDocument autoSrcDoc((inputFilesPath + "/" + strFile).c_str(), "rb");

		ret = autoSrcDoc.LoadPDFDocument(NULL);
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			continue;
		}

		CAutoDocument autoInsertDoc((inputFilesPath + "/" + "InsertPages.pdf").c_str(), "rb");
		ret = autoInsertDoc.LoadPDFDocument(NULL);
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			continue;
		}
		switch (icount){
			case 0:
				{
					//Insert srcDoc pages to insertDoc. And save.
					InsertPagesToDoc(autoInsertDoc.GetDocument(), autoSrcDoc.GetDocument());
					FSDK_SavePDFFile(autoInsertDoc.GetDocument(), (outputFile + strFileName + "_insert.pdf").c_str());
					break;
				}
			case 1:
				{
					//Split doc.
					Split_Doc(autoSrcDoc.GetDocument(), outputFile, strFileName);
					break;
				}
			case 2:
				{
					//Merge two doc.
					Merge_Doc(autoSrcDoc.GetDocument(), autoInsertDoc.GetDocument());
					FSDK_SavePDFFile(autoInsertDoc.GetDocument(), (outputFile + strFileName + "_merge.pdf").c_str());
					break;
				}
			case 3:
				{
					//Delete page
					Delete_Page(autoSrcDoc.GetDocument());
					FSDK_SavePDFFile(autoSrcDoc.GetDocument(), (outputFile + strFileName + "_delete.pdf").c_str());
					break;
				}
			case 4:
				{
					//Set pageindex
					SetIndex_Page(autoSrcDoc.GetDocument());
					FSDK_SavePDFFile(autoSrcDoc.GetDocument(),(outputFile + strFileName + "_setPageIndex.pdf").c_str());
					break;
				}
		}
	}
}

void InsertFoldPage(void* folder, string inputFilesPath, string outputFilesPath)
{
	FSDK_OutputLog("success\r\n");
	do
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		string outputFile = outputFilesPath + "/";
		if (strFile.size() > 0 && !isDir)
		{
			if (FSDK_CompareExtName(strFile.c_str(), ".pdf") && strcmp(strFile.c_str(), "InsertPages.pdf"))
			{
				//Get file name without file ext.
				InsertFilePage(strFile, inputFilesPath, outputFile);
			}
		}
		if (strFile.size() < 1) break;
		if (isDir) continue;
	} while (1);
}

int main(int argc, char* argv[])
{
	FSDK_SetExecuteFilePath(argc, argv);

	//Get log file.
	string strlogPath = FSDK_GetLogFile("pdfpage_organization");
	FSDK_OpenLog(strlogPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfpage_organization\r\n\r\n");

	//Init library.
	FS_BOOL bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}

	//Init PDF module.
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Get output file path.
	string outputFilesPath = FSDK_GetOutputFilesFolder("pdfpage_organization");
	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFilesPath + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Get input file path.
	string inputFilesPath = FSDK_GetInputFilesFolder("pdfpage_organization");
	void* folder = FSDK_OpenFolder(inputFilesPath.c_str(), FALSE);

	if (folder)
	{
		FSDK_OutputLog("Success: Initialize the PDF module successfully.\r\n");
		//Get input file path.
		string inputFilesPath = FSDK_GetInputFilesFolder("pdfpage_organization");
		void* folder = FSDK_OpenFolder(inputFilesPath.c_str(), FALSE);
		//Get output file path.
		string outputFilesPath = FSDK_GetOutputFilesFolder("pdfpage_organization");
		if (folder)
		{
			FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfpage_organization: ");
			FSDK_OutputLog("Input File from folder: %s\r\n", inputFilesPath.c_str());
			FSDK_OutputLog("Output File to folder: %s\r\n", outputFilesPath.c_str());
			InsertFoldPage(folder, inputFilesPath, outputFilesPath);
			//Close access to specific folder.
			FSDK_CloseFolder(folder);
		}
		else
		{
			string inputFilesPath = FSDK_GetFixFolder();
			string filename = "SourcePage.pdf";
			string outputFile = outputFilesPath + "/";
			FSDK_OutputLog("Input from folder: %s\r\n", inputFilesPath.c_str());
			FSDK_OutputLog("Output to folder: %s\r\n", outputFile.c_str());
			InsertFilePage(filename, inputFilesPath, outputFile);
		}

		FSDK_OutputLog("\r\nFinalize PDF module");
		//Finalize PDF module.
		FSCRT_PDFModule_Finalize();

		FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfpage_organization: ");
		InsertFoldPage(folder, inputFilesPath, outputFilesPath);
		//Close access to specific folder.
		FSDK_CloseFolder(folder);

	}
	else
	{
		string inputFilesPath = FSDK_GetFixFolder();
		string filename = "SourcePage.pdf";
		string outputFile = outputFilesPath + "/";
		InsertFilePage(filename, inputFilesPath, outputFile);
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}
